package com.adealink.weparty.module.widget.form.data

import com.adealink.frame.aab.util.getCompatString
import com.adealink.frame.base.IError
import com.adealink.weparty.R as APP_R

/**
 * 表格控件初始化异常
 */
sealed class FieldException(override val message: String) : Exception(message)
class UnSupportFieldDataException(val errorMsg: String) : FieldException(
    getCompatString(APP_R.string.common_widget_unsupported_field, errorMsg)
)

/**
 * 必填项没填写
 */
sealed class FieldValidateError(override val msg: String) : IError(msg = msg)

class RequireValueNullError(override val msg: String = "") : FieldValidateError(msg)

class NumberLessThanMinError(override val msg: String = "") : FieldValidateError(msg)
class NumberMoreThanMaxError(override val msg: String = "") : FieldValidateError(msg)


class ArrayLessThanMinError(override val msg: String = "") : FieldValidateError(msg)
class ArrayMoreThanMaxError(override val msg: String = "") : FieldValidateError(msg)


class StringLessThanMinError(override val msg: String = "") : FieldValidateError(msg)
class StringMoreThanMaxError(override val msg: String = "") : FieldValidateError(msg)